#include "common_defs.h"
#include "Morse.h"

/***************************************************************************
 *                                                     *
 ***************************************************************************/

UINT16 volatile InterruptsCounter = 0;

/***************************************************************************
 *                                                 *
 ***************************************************************************/
const UINT8 PROGMEM text[] = "                    VVV VVV VVV BEACON DE UA0LTB 10WATT ANT DIPOLE QTH LOC PN53XC VLADIVOSTOK RUSSIA \0";

/***************************************************************************
 *   *
 ***************************************************************************/
void PTT_ON(void)
{
   PORT &= ~_BV(PIN_PTT);  /*   (PB3 = 0) */
   PORT &= ~_BV(PIN_PTT2); /*   (PB2 = 0) */
}

/****************************************************************************
 *   *
 ****************************************************************************/
void PTT_OFF(void)
{
   PORT  |=  _BV(PIN_PTT);  /* PB3    1 */
   PORT  |=  _BV(PIN_PTT2); /* PB2    1 */
}

/****************************************************************************
 *                                                  *
 ****************************************************************************/
void Wait(UINT16 delay)
{
   for (;;)
   {
      sleep_mode();
      if (InterruptsCounter > delay)
      {
         InterruptsCounter = 0;
         break;
      }
   }
}

/****************************************************************************
 *   /                                 *
 ****************************************************************************/
void IOinit(void)
{
   cli();    //  .       .
             //  -   ! */

   ACSR      = _BV(ACD);                     //     

   TCCR1    &= ~(_BV(COM1A1)|_BV(COM1A0));   // / 1    OC1A
   GTCCR    &= ~(_BV(COM1B1)|_BV(COM1B0));   // / 1    OC1B
   PLLCSR   &= ~_BV(PCKE); // If this bit is cleared, system clock CK is used as Timer/Counter1 clock source
   TIMSK    |= _BV(TOIE1);                   //    - 1

   PORT_DIR |= _BV(PIN_PTT)|_BV(PIN_PTT2);   //  PIN_PTT  PIN_PTT2   
   PORT_DIR &= ~_BV(PIN_MODE);               //  PIN_MODE  -  

   PORT     |= _BV(PIN_MODE)|_BV(PIN_PTT)|_BV(PIN_PTT2); // PIN_PTT, PIN_PTT2 -  1,     PIN_MODE
	
   stopTCNT1();

   sei();
}

/****************************************************************************
 *                                M A I N                                   *
 ****************************************************************************/
int main(void)
{
   UINT8         symbol_code;
   UINT8         symbol_length;
   UINT16        letter;
   UINT16        current_delay;
   const UINT8  *p_text;

   IOinit();

   PTT_OFF();

   startTCNT1();

   Wait(MORSE_DASH_LENGTH); //     

   if ((~PORT_PIN & _BV(PIN_MODE)))  //  ,     PIN_MODE
   {
      PORT &= ~_BV(PIN_PTT);       //   (PB3 = 0)
      PORT &= ~_BV(PIN_PTT2);      //   (PB2 = 0)
		do {sleep_mode();} while(1); //  ,    :)
   }

   p_text = &text[0] + 20; // 20      

   for (;;)
   {
      for(;;)
      {
         letter = (UINT16)__LPM(p_text++);
         if (!letter) break;

         if (SPACE != letter)
         {
            letter        = (letter - EXCLAMATION_MARK)<<1;
            symbol_code   = __LPM((UINT16)(&morse_lookup_table[0]+letter++));   //p_morse_lookup_table[letter].symbol_code;
            symbol_length = __LPM((UINT16)(&morse_lookup_table[0]+letter));     //p_morse_lookup_table[letter].symbol_length;

            //   
            for (; symbol_length > 0; --symbol_length)
            {
               PTT_ON();
               if (symbol_code & 0x80)     //      
                  current_delay = MORSE_DOT_LENGTH;
               else
                  current_delay = MORSE_DASH_LENGTH;

               symbol_code <<= 1;

               Wait(current_delay);         //     
               PTT_OFF();

               Wait(MORSE_PAUSE_WITHIN_SIGN); //    
            }
            Wait(MORSE_PAUSE_BETWEEN_SIGNES); //   
         }
         else
         {
            Wait(MORSE_PAUSE_BETWEEN_WORDS);  //     
         }
      }

      //  "" 3 
      PTT_ON();
      Wait(745);
      PTT_OFF();

      p_text = &text[0];
   }
   return 0;
}

/****************************************************************************
 *     - 1                 *
 *  PWM-,    Timer/Counter1    OCR1A  *
 * ,    (SREG I-bit = 1 and TOIE1 = 1)      *
 *    TIMER1_OVF_vect.                              *
 ****************************************************************************/
ISR(TIMER1_OVF_vect)
{
   InterruptsCounter++;
}

/*****************************
 *        *
 *****************************/
void Wait_us(UINT16 pause)
{
   UINT16 i;
   for (i = 0; i < pause; i++)
   {
      __asm__ volatile("nop");
      __asm__ volatile("nop");
      __asm__ volatile("nop");
      __asm__ volatile("nop");
      __asm__ volatile("nop");
      __asm__ volatile("nop");
      __asm__ volatile("nop");
      __asm__ volatile("nop");
   }
}

/*****************************
 *        *
 *****************************/
void Wait_ms(UINT16 pause)
{
   UINT16 i;
   for (i = 0; i < pause; i++)
   {
      Wait_us(1000);
   }
}

/*********************************************************************
 *   / 1.          *
 *  +----+----+----+----+--------------------------------------------
 *  CS13CS12CS11CS10                                    
 *  +----+----+----+-------------------------------------------------+
 *  0   0   0   0   / 1                  
 *  +----+----+----+-------------------------------------------------+
 *  0   0   0   1   CK                                          
 *  +----+----+----+-------------------------------------------------+
 *  0   0   1   0   CK/2                                        
 *  +----+----+----+-------------------------------------------------+
 *  0   0   1   1   CK/4                                        
 *  +----+----+----+-------------------------------------------------+
 *  0   1   0   0   CK/8                                        
 *  +----+----+----+-------------------------------------------------+
 *  0   1   0   1   CK/16                                       
 *  +----+----+----+-------------------------------------------------+
 *  0   1   1   0   CK/32                                       
 *  +----+----+----+-------------------------------------------------+
 *  0   1   1   1   CK/64                                       
 *  +----+----+----+-------------------------------------------------+
 *  1   0   0   0   CK/128                                      
 *  +----+----+----+-------------------------------------------------+
 *  1   0   0   1   CK/256                                      
 *  +----+----+----+-------------------------------------------------+
 *  1   0   1   0   CK/512                                      
 *  +----+----+----+-------------------------------------------------+
 *  1   0   1   1   CK/1024                                     
 *  +----+----+----+-------------------------------------------------+
 *  1   1   0   0   CK/2048                                     
 *  +----+----+----+-------------------------------------------------+
 *  1   1   0   1   CK/4096                                     
 *  +----+----+----+-------------------------------------------------+
 *  1   1   1   0   CK/8192                                     
 *  +----+----+----+-------------------------------------------------+
 *  1   1   1   1   CK/16384                                    
 *  +----+----+----+----+---------------------------------------------
 *********************************************************************/
void startTCNT1(void)
{
   cli();
   TIFR  |= _BV(TOV1);          //    - 1,   
   //TCCR1 |= (_BV(CS13) | _BV(CS12) | _BV(CS11) | _BV(CS10)); //  = 16384 + 
   //TCCR1 |= _BV(CS10);          //  = 1 + 
	TCCR1 |= _BV(CS13); //  = 128 + 
   TCNT1  = 0x00;
   sei();
}

/*************************************************
 *   / 1                      *
 *************************************************/
void stopTCNT1(void)
{
   cli();
   TIFR |= _BV(TOV1);            //    - 1,   
   TCCR1 &= ~(_BV(CS13) | _BV(CS12) | _BV(CS11) | _BV(CS10));  //  = 0 , .. 
   sei();
}
